@@ -1,16 +1,24 @@
-import aiofiles
 import copy
 import fnmatch
-import msgpack
 import logging
 import pathlib
 import re
+from collections.abc import Mapping
+from collections.abc import MutableMapping
+from collections.abc import Sequence
+from typing import Any
+from typing import Dict
+from typing import Iterable
+from typing import Iterator
+from typing import List
+from typing import Set
+
+import aiofiles
+import msgpack
 import yaml
 
-from . import DEFAULT_TARGET_DELIM
 from . import args
-from collections.abc import Mapping, MutableMapping, Sequence
-from typing import Any, Dict, Iterable, List, Set, Iterator
+from . import DEFAULT_TARGET_DELIM
 
 log = logging.getLogger(__name__)
 
@@ -132,7 +140,7 @@ class NamespaceDict(dict):
     A dictionary that can access it's string keys through the namespace
     """
 
-    def __init__(self, seq: Iterable = None, **kwargs):
+    def __init__(self, seq: Mapping = None, **kwargs):
         """
         NamespaceDict() -> new empty namespaced dictionary
         NamespaceDict(mapping) -> new namespaced dictionary initialized from a mapping object's
@@ -144,15 +152,28 @@ class NamespaceDict(dict):
         NamespaceDict(**kwargs) -> new namespaced dictionary initialized with the name=value pairs
             in the keyword argument list.  For example:  NamespaceDict(one=1, two=2)
         """
-        super().__init__()
-        if seq is not None:
-            for k in seq:
-                self[k] = seq[k]
+        if seq:
+            if isinstance(seq, Mapping):
+                super().__init__()
+                for k, v in seq.items():
+                    self[k] = v
+            else:
+                super().__init__(seq)
+        else:
+            super().__init__()
+
         for k, v in kwargs.items():
             self[k] = v
 
+    def __setitem__(self, k, v):
+        if isinstance(v, Mapping) and not isinstance(v, NamespaceDict):
+            v = NamespaceDict(v)
+            self[k] = v
+        else:
+            super().__setitem__(k, v)
+
     def __setattr__(self, k: str, v: Any):
-        if isinstance(v, Iterable) and not isinstance(v, NamespaceDict):
+        if isinstance(v, Mapping) and not isinstance(v, NamespaceDict):
             v = NamespaceDict(v)
         self[k] = v
 
@@ -162,7 +183,7 @@ class NamespaceDict(dict):
         return self[k]
 
     def __copy__(self):
-        return NamespaceDict(self.copy())
+        return {k: copy.copy(v) for k, v in self.items()}
 
     def __deepcopy__(self, memodict=None):
         if memodict is None:
@@ -307,7 +328,7 @@ def compare_dicts(old: Dict = None, new: Dict = None) -> Dict[str, Dict]:
     dict describing the changes that were made.
     """
     ret = {}
-    for key in set((new or {})).union((old or {})):
+    for key in set(new or {}).union(old or {}):
         if key not in old:
             # New key
             ret[key] = {"old": "", "new": new[key]}
